@@ -56,17 +56,13 @@ pub(crate) mod type1;
 pub(crate) mod type2;
 pub mod value;
 
-pub use crate::{
-    namespace::{AmlHandle, AmlName, Namespace},
-    value::AmlValue,
-};
+pub use crate::{namespace::*, value::AmlValue};
 
 use alloc::{boxed::Box, string::ToString};
 use core::mem;
 use log::{error, warn};
 use misc::{ArgNum, LocalNum};
 use name_object::Target;
-use namespace::LevelType;
 use parser::{Parser, Propagate};
 use pkg_length::PkgLength;
 use term_object::term_list;
@@ -91,19 +87,9 @@ pub enum DebugVerbosity {
 }
 
 struct MethodContext {
-    /*
-     * AML local variables. These are used when we invoke a control method. A `None` value
-     * represents a null AML object.
-     */
-    local_0: Option<AmlValue>,
-    local_1: Option<AmlValue>,
-    local_2: Option<AmlValue>,
-    local_3: Option<AmlValue>,
-    local_4: Option<AmlValue>,
-    local_5: Option<AmlValue>,
-    local_6: Option<AmlValue>,
-    local_7: Option<AmlValue>,
-
+    /// AML local variables. These are used when we invoke a control method. A `None` value represents a null AML
+    /// object.
+    locals: [Option<AmlValue>; 8],
     /// If we're currently invoking a control method, this stores the arguments that were passed to
     /// it. It's `None` if we aren't invoking a method.
     args: Args,
@@ -111,17 +97,11 @@ struct MethodContext {
 
 impl MethodContext {
     fn new(args: Args) -> MethodContext {
-        MethodContext {
-            local_0: None,
-            local_1: None,
-            local_2: None,
-            local_3: None,
-            local_4: None,
-            local_5: None,
-            local_6: None,
-            local_7: None,
-            args,
-        }
+        // XXX: this is required because `Option<AmlValue>` is not `Copy`, so it can't be used to initialize an
+        // array, but consts can :(
+        const NONE_BUT_CONST: Option<AmlValue> = None;
+
+        MethodContext { locals: [NONE_BUT_CONST; 8], args }
     }
 }
 
@@ -205,6 +185,8 @@ impl AmlContext {
                             // If the method doesn't return a value, we implicitly return `0`
                             Ok(_) => Ok(AmlValue::Integer(0)),
                             Err((_, _, Propagate::Return(result))) => Ok(result),
+                            Err((_, _, Propagate::Break)) => Err(AmlError::BreakInInvalidPosition),
+                            Err((_, _, Propagate::Continue)) => Err(AmlError::ContinueInInvalidPosition),
                             Err((_, _, Propagate::Err(err))) => {
                                 error!("Failed to execute control method: {:?}", err);
                                 Err(err)
@@ -250,7 +232,6 @@ impl AmlContext {
     // TODO: docs
     pub fn initialize_objects(&mut self) -> Result<(), AmlError> {
         use name_object::NameSeg;
-        use namespace::NamespaceLevel;
         use value::StatusObject;
 
         /*
@@ -306,6 +287,19 @@ impl AmlContext {
         Ok(())
     }
 
+    pub(crate) fn read_target(&self, target: &Target) -> Result<&AmlValue, AmlError> {
+        match target {
+            Target::Null => todo!(),
+            Target::Name(name) => {
+                let (_, handle) = self.namespace.search(name, &self.current_scope)?;
+                self.namespace.get(handle)
+            }
+            Target::Debug => todo!(),
+            Target::Arg(arg) => self.current_arg(*arg),
+            Target::Local(local) => self.local(*local),
+        }
+    }
+
     /// Get the value of an argument by its argument number. Can only be executed from inside a control method.
     pub(crate) fn current_arg(&self, arg: ArgNum) -> Result<&AmlValue, AmlError> {
         self.method_context.as_ref().ok_or(AmlError::NotExecutingControlMethod)?.args.arg(arg)
@@ -316,42 +310,39 @@ impl AmlContext {
         if let None = self.method_context {
             return Err(AmlError::NotExecutingControlMethod);
         }
-
-        match local {
-            0 => self.method_context.as_ref().unwrap().local_0.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            1 => self.method_context.as_ref().unwrap().local_1.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            2 => self.method_context.as_ref().unwrap().local_2.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            3 => self.method_context.as_ref().unwrap().local_3.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            4 => self.method_context.as_ref().unwrap().local_4.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            5 => self.method_context.as_ref().unwrap().local_5.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            6 => self.method_context.as_ref().unwrap().local_6.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            7 => self.method_context.as_ref().unwrap().local_7.as_ref().ok_or(AmlError::InvalidLocalAccess(local)),
-            _ => Err(AmlError::InvalidLocalAccess(local)),
+        if local > 7 {
+            return Err(AmlError::InvalidLocalAccess(local));
         }
+
+        self.method_context.as_ref().unwrap().locals[local as usize]
+            .as_ref()
+            .ok_or(AmlError::InvalidLocalAccess(local))
     }
 
-    /// Perform a store into a `Target`. This returns a value read out of the target, if neccessary, as values can
-    /// be altered during a store in some circumstances. If the target is a `Name`, this also performs required
-    /// implicit conversions. Stores to other targets are semantically equivalent to a `CopyObject`.
+    /// Perform a store into a `Target`, according to the rules specified by §19.3.5.8. This returns a value read
+    /// out of the target, if neccessary, as values can be altered during a store in some circumstances.  When
+    /// required, this also performs required implicit conversions, otherwise stores are semantically equivalent to
+    /// a `CopyObject`.
     pub(crate) fn store(&mut self, target: Target, value: AmlValue) -> Result<AmlValue, AmlError> {
         match target {
             Target::Name(ref path) => {
                 let (_, handle) = self.namespace.search(path, &self.current_scope)?;
-                let converted_object = match self.namespace.get(handle).unwrap().type_of() {
-                    /*
-                     * We special-case FieldUnits here because we don't have the needed information to actually do
-                     * the write if we try and convert using `as_type`.
-                     */
+
+                match self.namespace.get(handle).unwrap().type_of() {
                     AmlType::FieldUnit => {
                         let mut field = self.namespace.get(handle).unwrap().clone();
                         field.write_field(value, self)?;
-                        field.read_field(self)?
+                        field.read_field(self)
                     }
-                    typ => value.as_type(typ, self)?,
-                };
-
-                *self.namespace.get_mut(handle)? = converted_object;
-                Ok(self.namespace.get(handle)?.clone())
+                    AmlType::BufferField => {
+                        log::info!("Trying to write to buffer field!");
+                        todo!()
+                    }
+                    typ => {
+                        *self.namespace.get_mut(handle)? = value.as_type(typ, self)?;
+                        Ok(self.namespace.get(handle)?.clone())
+                    }
+                }
             }
 
             Target::Debug => {
@@ -364,15 +355,13 @@ impl AmlContext {
                     return Err(AmlError::NotExecutingControlMethod);
                 }
 
-                match arg_num {
-                    1 => self.method_context.as_mut().unwrap().args.arg_1 = Some(value.clone()),
-                    2 => self.method_context.as_mut().unwrap().args.arg_2 = Some(value.clone()),
-                    3 => self.method_context.as_mut().unwrap().args.arg_3 = Some(value.clone()),
-                    4 => self.method_context.as_mut().unwrap().args.arg_4 = Some(value.clone()),
-                    5 => self.method_context.as_mut().unwrap().args.arg_5 = Some(value.clone()),
-                    6 => self.method_context.as_mut().unwrap().args.arg_6 = Some(value.clone()),
-                    _ => return Err(AmlError::InvalidArgAccess(arg_num)),
-                }
+                /*
+                 * Stores into `Arg` objects are simply copied with no conversion applied, unless the `Arg`
+                 * contains an Object Reference, in which case an automatic de-reference occurs and the object is
+                 * copied to the target of the Object Reference, instead of overwriting the `Arg.`
+                 */
+                // TODO: implement behaviour for object references
+                self.method_context.as_mut().unwrap().args.store_arg(arg_num, value.clone())?;
                 Ok(value)
             }
 
@@ -381,17 +370,11 @@ impl AmlContext {
                     return Err(AmlError::NotExecutingControlMethod);
                 }
 
-                match local_num {
-                    0 => self.method_context.as_mut().unwrap().local_0 = Some(value.clone()),
-                    1 => self.method_context.as_mut().unwrap().local_1 = Some(value.clone()),
-                    2 => self.method_context.as_mut().unwrap().local_2 = Some(value.clone()),
-                    3 => self.method_context.as_mut().unwrap().local_3 = Some(value.clone()),
-                    4 => self.method_context.as_mut().unwrap().local_4 = Some(value.clone()),
-                    5 => self.method_context.as_mut().unwrap().local_5 = Some(value.clone()),
-                    6 => self.method_context.as_mut().unwrap().local_6 = Some(value.clone()),
-                    7 => self.method_context.as_mut().unwrap().local_7 = Some(value.clone()),
-                    _ => return Err(AmlError::InvalidLocalAccess(local_num)),
-                }
+                /*
+                 * Stores into `Local` objects are always simply copied into the destination with no conversion
+                 * applied, even if it contains an Object Reference.
+                 */
+                self.method_context.as_mut().unwrap().locals[local_num as usize] = Some(value.clone());
                 Ok(value)
             }
 
@@ -701,6 +684,10 @@ pub trait Handler: Send + Sync {
     fn write_pci_u8(&self, segment: u16, bus: u8, device: u8, function: u8, offset: u16, value: u8);
     fn write_pci_u16(&self, segment: u16, bus: u8, device: u8, function: u8, offset: u16, value: u16);
     fn write_pci_u32(&self, segment: u16, bus: u8, device: u8, function: u8, offset: u16, value: u32);
+
+    fn handle_fatal_error(&self, fatal_type: u8, fatal_code: u32, fatal_arg: u64) {
+        panic!("Fatal error while executing AML (encountered DefFatal op). fatal_type = {:?}, fatal_code = {:?}, fatal_arg = {:?}", fatal_type, fatal_code, fatal_arg);
+    }
 }
 
 #[derive(Clone, PartialEq, Eq, Debug)]
@@ -726,6 +713,8 @@ pub enum AmlError {
     /// that parser (e.g. the wrong opcode starts the stream). This is handled specially by some
     /// parsers such as `or` and `choice!`.
     WrongParser,
+    /// Returned when a `DefFatal` op is encountered. This is separately reported using [`Handler::handle_fatal_error`].
+    FatalError,
 
     /*
      * Errors produced manipulating AML names.
@@ -759,6 +748,12 @@ pub enum AmlError {
     InvalidArgAccess(ArgNum),
     /// Produced when a method accesses a local that it has not stored into.
     InvalidLocalAccess(LocalNum),
+    /// Tried to invoke a method with too many arguments.
+    TooManyArgs,
+    /// A `DefBreak` operation was performed outside of a `DefWhile` or `DefSwitch`.
+    BreakInInvalidPosition,
+    /// A `DefContinue` operation was performed outside of a `DefWhile`.
+    ContinueInInvalidPosition,
 
     /*
      * Errors produced parsing the PCI routing tables (_PRT objects).
