@@ -26,8 +26,15 @@ pub const DEF_BUFFER_OP: u8 = 0x11;
 pub const DEF_PACKAGE_OP: u8 = 0x12;
 pub const DEF_METHOD_OP: u8 = 0x14;
 pub const DEF_EXTERNAL_OP: u8 = 0x15;
+pub const DEF_CREATE_DWORD_FIELD_OP: u8 = 0x8a;
+pub const DEF_CREATE_WORD_FIELD_OP: u8 = 0x8b;
+pub const DEF_CREATE_BYTE_FIELD_OP: u8 = 0x8c;
+pub const DEF_CREATE_BIT_FIELD_OP: u8 = 0x8d;
+pub const DEF_CREATE_QWORD_FIELD_OP: u8 = 0x8f;
 pub const EXT_DEF_MUTEX_OP: u8 = 0x01;
+pub const EXT_DEF_CREATE_FIELD_OP: u8 = 0x13;
 pub const EXT_REVISION_OP: u8 = 0x30;
+pub const EXT_DEF_FATAL_OP: u8 = 0x32;
 pub const EXT_DEF_OP_REGION_OP: u8 = 0x80;
 pub const EXT_DEF_FIELD_OP: u8 = 0x81;
 pub const EXT_DEF_DEVICE_OP: u8 = 0x82;
@@ -38,10 +45,13 @@ pub const EXT_DEF_THERMAL_ZONE_OP: u8 = 0x85;
 /*
  * Type 1 opcodes
  */
+pub const DEF_CONTINUE_OP: u8 = 0x9f;
 pub const DEF_IF_ELSE_OP: u8 = 0xa0;
 pub const DEF_ELSE_OP: u8 = 0xa1;
+pub const DEF_WHILE_OP: u8 = 0xa2;
 pub const DEF_NOOP_OP: u8 = 0xa3;
 pub const DEF_RETURN_OP: u8 = 0xa4;
+pub const DEF_BREAK_OP: u8 = 0xa5;
 pub const DEF_BREAKPOINT_OP: u8 = 0xcc;
 
 /*
@@ -50,6 +60,8 @@ pub const DEF_BREAKPOINT_OP: u8 = 0xcc;
 pub const DEF_STORE_OP: u8 = 0x70;
 pub const DEF_ADD_OP: u8 = 0x72;
 pub const DEF_CONCAT_OP: u8 = 0x73;
+pub const DEF_INCREMENT_OP: u8 = 0x75;
+pub const DEF_DECREMENT_OP: u8 = 0x76;
 pub const DEF_SHIFT_LEFT: u8 = 0x79;
 pub const DEF_SHIFT_RIGHT: u8 = 0x7a;
 pub const DEF_AND_OP: u8 = 0x7b;
