@@ -26,7 +26,7 @@ use crate::{
     AmlHandle,
     DebugVerbosity,
 };
-use alloc::string::String;
+use alloc::{string::String, sync::Arc, vec::Vec};
 use core::str;
 
 /// `TermList`s are usually found within explicit-length objects (so they have a `PkgLength`
@@ -86,10 +86,9 @@ where
     'c: 'a,
 {
     /*
-     * NamedObj := DefBankField | DefCreateBitField | DefCreateByteField | DefCreateDWordField |
-     *             DefCreateField | DefCreateQWordField | DefCreateWordField | DefDataRegion |
-     *             DefExternal | DefOpRegion | DefPowerRes | DefProcessor | DefThermalZone |
-     *             DefMethod | DefMutex
+     * NamedObj := DefBankField | DefCreateBitField | DefCreateByteField | DefCreateWordField | DefCreateDWordField |
+     *             DefCreateQWordField | DefCreateField | DefDataRegion | DefExternal | DefOpRegion | DefPowerRes |
+     *             DefProcessor | DefThermalZone | DefMethod | DefMutex
      *
      * XXX: DefMethod and DefMutex (at least) are not included in any rule in the AML grammar,
      * but are defined in the NamedObj section so we assume they're part of NamedObj
@@ -98,6 +97,12 @@ where
         DebugVerbosity::AllScopes,
         "NamedObj",
         choice!(
+            def_create_bit_field(),
+            def_create_byte_field(),
+            def_create_word_field(),
+            def_create_dword_field(),
+            def_create_qword_field(),
+            def_create_field(),
             def_op_region(),
             def_field(),
             def_method(),
@@ -173,6 +178,212 @@ where
         .discard_result()
 }
 
+pub fn def_create_bit_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateBitField := 0x8d SourceBuf BitIndex NameString
+     * SourceBuf := TermArg => Buffer
+     * BitIndex := TermArg => Integer
+     */
+    opcode(opcode::DEF_CREATE_BIT_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefCreateBitField",
+            term_arg().then(term_arg()).then(name_string()).map_with_context(
+                |((source, index), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index, length: 1 }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
+pub fn def_create_byte_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateByteField := 0x8c SourceBuf ByteIndex NameString
+     * SourceBuf := TermArg => Buffer
+     * ByteIndex := TermArg => Integer
+     */
+    opcode(opcode::DEF_CREATE_BYTE_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefCreateByteField",
+            term_arg().then(term_arg()).then(name_string()).map_with_context(
+                |((source, index), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index * 8, length: 8 }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
+pub fn def_create_word_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateWordField := 0x8b SourceBuf ByteIndex NameString
+     * SourceBuf := TermArg => Buffer
+     * ByteIndex := TermArg => Integer
+     */
+    opcode(opcode::DEF_CREATE_WORD_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefCreateWordField",
+            term_arg().then(term_arg()).then(name_string()).map_with_context(
+                |((source, index), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index * 8, length: 16 }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
+pub fn def_create_dword_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateDWordField := 0x8a SourceBuf ByteIndex NameString
+     * SourceBuf := TermArg => Buffer
+     * ByteIndex := TermArg => Integer
+     */
+    opcode(opcode::DEF_CREATE_DWORD_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefCreateDWordField",
+            term_arg().then(term_arg()).then(name_string()).map_with_context(
+                |((source, index), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index * 8, length: 32 }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
+pub fn def_create_qword_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateQWordField := 0x8f SourceBuf ByteIndex NameString
+     * SourceBuf := TermArg => Buffer
+     * ByteIndex := TermArg => Integer
+     */
+    opcode(opcode::DEF_CREATE_QWORD_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefCreateQWordField",
+            term_arg().then(term_arg()).then(name_string()).map_with_context(
+                |((source, index), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index * 8, length: 64 }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
+pub fn def_create_field<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefCreateField := ExtOpPrefix 0x13 SourceBuf BitIndex NumBits NameString
+     * SourceBuf := TermArg => Buffer
+     * BitIndex := TermArg => Integer
+     * NumBits := TermArg => Integer
+     */
+    ext_opcode(opcode::EXT_DEF_CREATE_FIELD_OP)
+        .then(comment_scope(
+            DebugVerbosity::Scopes,
+            "DefCreateField",
+            term_arg().then(term_arg()).then(term_arg()).then(name_string()).map_with_context(
+                |(((source, index), num_bits), name), context| {
+                    let source_data: Arc<Vec<u8>> = try_with_context!(context, source.as_buffer(context)).clone();
+                    let index = try_with_context!(context, index.as_integer(context));
+                    let num_bits = try_with_context!(context, num_bits.as_integer(context));
+
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value_at_resolved_path(
+                            name,
+                            &context.current_scope,
+                            AmlValue::BufferField { buffer_data: source_data, offset: index, length: num_bits }
+                        )
+                    );
+
+                    (Ok(()), context)
+                },
+            ),
+        ))
+        .discard_result()
+}
+
 pub fn def_op_region<'a, 'c>() -> impl Parser<'a, 'c, ()>
 where
     'c: 'a,
