@@ -20,10 +20,11 @@ use crate::{
 };
 use alloc::{
     string::{String, ToString},
+    sync::Arc,
     vec,
     vec::Vec,
 };
-use core::{cmp::Ordering, convert::TryInto, mem};
+use core::{cmp::Ordering, convert::TryInto, mem, ops::Deref};
 
 /// Type 2 opcodes return a value and so can be used in expressions.
 pub fn type2_opcode<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
@@ -37,7 +38,7 @@ where
      *                DefLAnd | DefLEqual | DefLGreater | DefLGreaterEqual | DefLLess | DefLLessEqual |
      *                DefMid | DefLNot | DefLNotEqual | DefLoadTable | DefLOr | DefMatch | DefMod |
      *                DefMultiply | DefNAnd | DefNOr | DefNot | DefObjectType | DefOr | DefPackage |
-     *                DefVarPackage | DefRefOf | DefShiftLeft | DefShitRight | DefSizeOf | DefStore |
+     *                DefVarPackage | DefRefOf | DefShiftLeft | DefShiftRight | DefSizeOf | DefStore |
      *                DefSubtract | DefTimer | DefToBCD | DefToBuffer | DefToDecimalString |
      *                DefToHexString | DefToInteger | DefToString | DefWait | DefXOr | MethodInvocation
      */
@@ -50,6 +51,8 @@ where
             def_buffer(),
             def_concat(),
             def_concat_res(),
+            def_increment(),
+            def_decrement(),
             def_l_equal(),
             def_l_greater(),
             def_l_greater_equal(),
@@ -151,7 +154,7 @@ where
                 })
             }),
         ))
-        .map(|((), buffer)| Ok(AmlValue::Buffer(buffer)))
+        .map(|((), buffer)| Ok(AmlValue::Buffer(Arc::new(buffer))))
 }
 
 pub fn def_concat<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
@@ -175,11 +178,12 @@ where
                         buffer.extend_from_slice(&left.to_le_bytes());
                         buffer.extend_from_slice(&right.to_le_bytes());
 
-                        AmlValue::Buffer(buffer)
+                        AmlValue::Buffer(Arc::new(buffer))
                     }
-                    AmlValue::Buffer(mut left) => {
-                        left.extend(try_with_context!(context, right.as_buffer(context)));
-                        AmlValue::Buffer(left)
+                    AmlValue::Buffer(left) => {
+                        let mut new = left.deref().clone();
+                        new.extend(try_with_context!(context, right.as_buffer(context)).iter());
+                        AmlValue::Buffer(Arc::new(new))
                     }
                     AmlValue::String(left) => {
                         let right = match right.as_concat_type() {
@@ -246,7 +250,7 @@ where
                         result.iter().fold(0u8, |checksum, byte| checksum.wrapping_add(*byte)).wrapping_neg(),
                     );
 
-                    AmlValue::Buffer(result)
+                    AmlValue::Buffer(Arc::new(result))
                 };
 
                 try_with_context!(context, context.store(target, result.clone()));
@@ -256,6 +260,50 @@ where
         .map(|((), result)| Ok(result))
 }
 
+fn def_increment<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
+where
+    'c: 'a,
+{
+    /*
+     * DefIncrement := 0x75 SuperName
+     */
+    opcode(opcode::DEF_INCREMENT_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefIncrement",
+            super_name().map_with_context(|addend, context| {
+                let value = try_with_context!(context, context.read_target(&addend));
+                let value = try_with_context!(context, value.as_integer(context));
+                let new_value = AmlValue::Integer(value + 1);
+                try_with_context!(context, context.store(addend, new_value.clone()));
+                (Ok(new_value), context)
+            }),
+        ))
+        .map(|((), result)| Ok(result))
+}
+
+fn def_decrement<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
+where
+    'c: 'a,
+{
+    /*
+     * DefDecrement := 0x76 SuperName
+     */
+    opcode(opcode::DEF_DECREMENT_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefDecrement",
+            super_name().map_with_context(|minuend, context| {
+                let value = try_with_context!(context, context.read_target(&minuend));
+                let value = try_with_context!(context, value.as_integer(context));
+                let new_value = AmlValue::Integer(value - 1);
+                try_with_context!(context, context.store(minuend, new_value.clone()));
+                (Ok(new_value), context)
+            }),
+        ))
+        .map(|((), result)| Ok(result))
+}
+
 fn def_l_or<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
 where
     'c: 'a,
@@ -417,11 +465,11 @@ where
                         match source {
                             AmlValue::Buffer(bytes) => {
                                 if index >= bytes.len() {
-                                    Ok(AmlValue::Buffer(vec![]))
+                                    Ok(AmlValue::Buffer(Arc::new(vec![])))
                                 } else if (index + length) >= bytes.len() {
-                                    Ok(AmlValue::Buffer(bytes[index..].to_vec()))
+                                    Ok(AmlValue::Buffer(Arc::new(bytes[index..].to_vec())))
                                 } else {
-                                    Ok(AmlValue::Buffer(bytes[index..(index + length)].to_vec()))
+                                    Ok(AmlValue::Buffer(Arc::new(bytes[index..(index + length)].to_vec())))
                                 }
                             }
                             /*
@@ -604,7 +652,8 @@ where
             .feed(|(path, num_args)| {
                 n_of(term_arg(), num_args.unwrap_or(0) as usize).map_with_context(move |arg_list, context| {
                     if num_args.is_some() {
-                        let result = context.invoke_method(&path, Args::from_list(arg_list));
+                        let args = try_with_context!(context, Args::from_list(arg_list));
+                        let result = context.invoke_method(&path, args);
                         (Ok(try_with_context!(context, result)), context)
                     } else {
                         (Ok(try_with_context!(context, context.namespace.get_by_path(&path)).clone()), context)
