@@ -197,7 +197,14 @@ pub enum AmlValue {
         flags: MethodFlags,
         code: MethodCode,
     },
-    Buffer(Vec<u8>),
+    Buffer(Arc<Vec<u8>>),
+    BufferField {
+        buffer_data: Arc<Vec<u8>>,
+        /// In bits.
+        offset: u64,
+        /// In bits.
+        length: u64,
+    },
     Processor {
         id: u8,
         pblk_address: u32,
@@ -206,6 +213,7 @@ pub enum AmlValue {
     Mutex {
         sync_level: u8,
     },
+    // TODO: I think this will need to be `Arc`ed as well, as `Index` can be used on both Buffers and Packages
     Package(Vec<AmlValue>),
     PowerResource {
         system_level: u8,
@@ -245,6 +253,7 @@ impl AmlValue {
             AmlValue::Device => AmlType::Device,
             AmlValue::Method { .. } => AmlType::Method,
             AmlValue::Buffer(_) => AmlType::Buffer,
+            AmlValue::BufferField { .. } => AmlType::BufferField,
             AmlValue::Processor { .. } => AmlType::Processor,
             AmlValue::Mutex { .. } => AmlType::Mutex,
             AmlValue::Package(_) => AmlType::Package,
@@ -293,7 +302,7 @@ impl AmlValue {
         }
     }
 
-    pub fn as_buffer(&self, context: &AmlContext) -> Result<Vec<u8>, AmlError> {
+    pub fn as_buffer(&self, context: &AmlContext) -> Result<Arc<Vec<u8>>, AmlError> {
         match self {
             AmlValue::Buffer(ref bytes) => Ok(bytes.clone()),
             // TODO: implement conversion of String and Integer to Buffer
@@ -511,54 +520,45 @@ impl AmlValue {
     }
 }
 
-/// A control method can take up to 7 arguments, each of which can be an `AmlValue`.
-#[derive(Clone, Debug, Default)]
-pub struct Args {
-    pub arg_0: Option<AmlValue>,
-    pub arg_1: Option<AmlValue>,
-    pub arg_2: Option<AmlValue>,
-    pub arg_3: Option<AmlValue>,
-    pub arg_4: Option<AmlValue>,
-    pub arg_5: Option<AmlValue>,
-    pub arg_6: Option<AmlValue>,
-}
+/// A control method can take up to 7 arguments, each of which is an `AmlValue`.
+#[derive(Clone, Default, Debug)]
+pub struct Args(pub [Option<AmlValue>; 7]);
 
 impl Args {
-    pub fn from_list(mut list: Vec<AmlValue>) -> Args {
-        assert!(list.len() <= 7);
-        list.reverse();
-        Args {
-            arg_0: list.pop(),
-            arg_1: list.pop(),
-            arg_2: list.pop(),
-            arg_3: list.pop(),
-            arg_4: list.pop(),
-            arg_5: list.pop(),
-            arg_6: list.pop(),
+    pub fn from_list(list: Vec<AmlValue>) -> Result<Args, AmlError> {
+        use core::convert::TryInto;
+
+        if list.len() > 7 {
+            return Err(AmlError::TooManyArgs);
         }
+
+        let mut args: Vec<Option<AmlValue>> = list.into_iter().map(Option::Some).collect();
+        args.extend(core::iter::repeat(None).take(7 - args.len()));
+        Ok(Args(args.try_into().unwrap()))
     }
-    /// Get an argument by its `ArgNum`.
-    ///
-    /// ### Panics
-    /// Panics if passed an invalid argument number (valid argument numbers are `0..=6`)
-    pub fn arg(&self, num: ArgNum) -> Result<&AmlValue, AmlError> {
-        match num {
-            0 => self.arg_0.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            1 => self.arg_1.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            2 => self.arg_2.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            3 => self.arg_3.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            4 => self.arg_4.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            5 => self.arg_5.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            6 => self.arg_6.as_ref().ok_or(AmlError::InvalidArgAccess(num)),
-            _ => Err(AmlError::InvalidArgAccess(num)),
+
+    pub fn arg(&self, arg: ArgNum) -> Result<&AmlValue, AmlError> {
+        if arg > 6 {
+            return Err(AmlError::InvalidArgAccess(arg));
         }
+
+        self.0[arg as usize].as_ref().ok_or(AmlError::InvalidArgAccess(arg))
+    }
+
+    pub fn store_arg(&mut self, arg: ArgNum, value: AmlValue) -> Result<(), AmlError> {
+        if arg > 6 {
+            return Err(AmlError::InvalidArgAccess(arg));
+        }
+
+        self.0[arg as usize] = Some(value);
+        Ok(())
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{test_utils::*, AmlError};
+    use crate::test_utils::*;
     use core::cmp::Ordering;
 
     #[test]
